import java.util.*;


public class Actor 
{

	private CommunicationChannel communicationChannel;
	private boolean goalie;    

	// Konstruktor fr die Bildung von Instanzen der Klasse Actor
	public Actor(CommunicationChannel communicationChannel, boolean goalie)    
	{
		this.communicationChannel = communicationChannel;
		this.goalie = goalie;   
 	}    
 	
 	
 	// Anmeldung des Spielers beim Server;
 	// Methode gibt zudem dem Spieler seine vom Server zugeteilte SpielerNummer zurck
 	// und bestimmt die Spielrichtung, in die der Spieler zu spielen hat    
 	public String[] init(String teamName)    
 	{
 		// Anmeldung des Spielers beim Server
 		if(goalie){
			communicationChannel.send(new String("(init " + teamName + "(version 7.0)) (goalie)"));
			//System.out.println("i hate you, BUG! Where is my goalie?!?"); 
 		}
		else
			communicationChannel.send(new String("(init " + teamName + "(version 7.0))"));
		// System.out.println("ACTOR:initialising Player with rcssserver...");
 		
 		// Empfangen von messages vom Server
 		String reply = communicationChannel.receive(); 
 		 		    
 		String[] Identifikation = { "0" , "0" };        
 		
 		// Abfrage, ob die vom Server erhaltene message mit "(init" beginnt
 		if (reply.startsWith("(init"))        
 		{
 			// ownSide gibt die Spielfeldseite an, auf der sich der Spieler befindet (l oder r) an      
 			char ownSide  = reply.charAt(6);
 			
 			// dadurch laesst sich bestimmen, in welche Richtung der Spieler zu spielen hat
        	switch (ownSide)
        	{
        		// falls der Spieler sich auf der linken Spielfeldhaelfte befindet:
        		// Gegnertor: rechtes Tor
        		// Spielrichtung: nach rechts
        		case 'l':
        			Identifikation[0] = "r";
        			break;
        		
        		// falls der Spieler sich auf der rechten Spielfeldhaelfte befindet:
        		// Gegnertor: linkes Tor
        		// Spielrichtung: nach links
        		case 'r':
            		Identifikation[0] = "l";
            		break;            
 			}
 			
 			// der substring gibt die SpielerNummer (1,2, ... 11) an  
 			Identifikation[1] = reply.substring(8, 10).trim();
 		 
 		 	return(Identifikation);      	
  			
		} 
		      
 		else        
 		{
 			// falls bereits 11 Spieler angemeldet sind ...
 			// oder der init-Befehl fehlerhaft war ...
 			System.out.println("ACTOR:wrong \"(init)\" reply or too many players on team!!");   
 			System.exit(1);
 			
 			return(Identifikation);
 		}   
 	}
 	
 	// Methode liefert Information, ob es sich um ein SeeEvent oder BodyEvent handelt
 	public Event event()   
 	{
 		String msg;        
 		Event event = null; 
 		       
 		while(event == null) 
 		{
 			// warte auf eine eintreffende server message
 			msg = communicationChannel.receive();            
 			// es kann sich dabei um ein visual ("see") oder ein body ("sense body") event handeln            
 			
 			
 			if(msg.startsWith("(see")) 
 			{              
 				// es ist ein visual ("see") event             
 				event = new VisualEvent(msg);          
 			}
 			
 			
 			else if(msg.startsWith("(sense_body")) 
 			{
 				// es ist ein body ("sense_body") event                
 				event = new BodyEvent(msg);            


 			}

//**NEU**NEU**NEU***************************************************************************			
 			else if(msg.startsWith("(hear"))
 			{
 				//es ist ein ("hear") event
 				event = new HearEvent(msg);
 			}
 			
//** ENDE*NEU**NEU**NEU***************************************************************************	
 	
		} 
 		
 		// gibt die Art des events zurck       
 		return event;    
 	}
 	
 	
 	
 	
 	// Methode ermglicht, dass der Spieler zu einem bestimmten Ziel schiessen kann
 	public void pass(double power, ReferencePoint player, double[] targetCoord)
 	{
 		// berechne die Differenz in Koordinaten von Player und Ziel
 		double x = player.getX() - targetCoord[0];
 		double y = player.getY() - targetCoord[1];
 		// berechne den inversen tangens
 		double relDir = Math.toDegrees(Math.atan(y/x));
 		kick(power, relDir - player.direction());
 	}
 	
 	
 	// Methode gibt dem Server Informationen ber Sehfeld und Sehqualitaet	
 	public void changeView(String width, String quality)
 	{
 		//width = narrow | normal | wide & quality = high | low
 		communicationChannel.send(new String("(change_view "+ width + " " + quality + ")"));
 	}
 	
 	
 	// Methode, die dem Spieler den Impuls gibt, loszurennen
 	public void dash(double power)    
 	{
 		communicationChannel.send(new String("(dash " + power + ")"));    
 	}    
 	
 	
 	// Methode, die das Kicken in eine gegebene Richtung mit gegebener Kraft ermglicht
 	public void kick(double power, double direction)    
 	{        
 		communicationChannel.send(new String("(kick " + power + " " + direction + ")"));    
 	}
 	
 	// Methode platziert den Spieler auf einer bestimmten Position auf dem Feld
 	// nur vor dem kick off mglich
 	// und nach einem Tor
 	public void move(double x, double y)    
 	{
 		communicationChannel.send(new String("(move " + x + " " + y + ")"));    
 	}    
 	
 	// mit dieser Methode kann der Spieler sich auf dem Spielfeld drehen
 	public void turn(double moment)    
 	{
 		communicationChannel.send(new String("(turn " + moment + ")"));    
 	}
 	
 	
    public void catchball (double dir)
    {
            communicationChannel.send(new String("catch " + dir + ")"));
    } 
 } 
