import java.io.*;
import java.net.*;


//This class handles UDP network traffic from/to the rcssserver.
public class CommunicationChannel
{
	
    //socket for incoming, outgoing messages
    private DatagramSocket socket;
    
    //IP address of the server 
    InetAddress serverAddress;
    
    //port number of the server
    private int serverPort;

    //Creates a new <code>CommunicationChannel</code> instance.     
    public CommunicationChannel(String serverName, int serverPort) 
    {
    	
        try 
        {
            //construct datagram socket and
            //bind it to any available port on the local machine
            this.socket        = new DatagramSocket();
        
            //set a timeout to wait for a packet if receiving
            socket.setSoTimeout(5000);
        
            //set target address
            this.serverAddress = InetAddress.getByName(serverName);
        
            //set target port
            this.serverPort = serverPort;
            
        } 
        catch (SocketException se) 
        {
            System.err.println("COMCHANNEL: Could not create datagram socket.");
            System.exit(1);
        } 
        catch (UnknownHostException uhe) 
        {
            System.err.println("COMCHANNEL: Could not determine the server's IP address.");
            System.exit(1);
        }
    }


    //Sends a String to the server containing the message which has to be sent to the server.
     public void send(String message) 
     {
     	
        try 
        {
            //convert the message to a byte array
            byte[] buffer = message.getBytes();
        
            //create the packet from the buffer and set the server's IP address and port
            DatagramPacket packet =
                new DatagramPacket(buffer, buffer.length, serverAddress, serverPort);
        
            //send the packet
            socket.send(packet);
            
        }
        catch (IOException ioe) 
        {
            System.err.println("COMCHANNEL: Could not send datagram packet.");
        } 
    }



    //Receives a message from the server containing the message read from the socket    
    public String receive() 
    {
    	
        //allocate buffer for server message
        byte[] buffer = new byte[32768];
    
        //create datagram packet for receiving
        DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
    
        try 
        {
            //receive packet, the method blocks until a datagram arrives
            socket.receive(packet);
        
        } 
        catch (IOException ioe) 
        {
        	System.err.println("COMCHANNEL: Could not receive datagram packet.");
        }
        
        //get the server port for the following actions
        this.serverPort = packet.getPort();
        
        //return the message as string and cut off null characters & \n in buffer
        return new String(packet.getData()).substring(0, packet.getLength()-1)+" ";  
        
    }    
    
}

